#include <stdio.h>
#include "xmmx.h"

int main(void)
{
	int rval;
	mmx_t ma;
	mmx_t mb;
	mmx_t mc;

	/* Note: Use either mm_support() or mmx_ok(), not both */

	/* Get and announce return value of mm_support() */
	rval = mm_support();
	printf("Value returned from mm_support was %x (%d).", rval, rval);
	printf(" (Indicates MMX is%s available)\n\n",
		(rval & 0x1)? "" : " not");
	printf(" (Indicates Extended MMX is%s available)\n\n",
		(rval & 0x2)? "" : " not");
	fflush(stdout); fflush(stderr);

	/* Get and announce return value of mmx_ok()
	rval = mmx_ok();
	printf("Value returned from mmx_ok was %x (%d).", rval, rval);
	printf(" (Indicates MMX is%s available)\n\n",(rval)? "" : " not");
	fflush(stdout); fflush(stderr);
	*/

	/* Get and announce return value of xmmx_ok()
	rval = xmmx_ok();
	printf("Value returned from xmmx_ok was %x (%d).", rval, rval);
	printf(" (Indicates Extended MMX is%s available)\n\n",
		(rval)? "" : " not");
	fflush(stdout); fflush(stderr);
	*/


	if(rval) {
		/* PMAG *****************************************************/
		ma.q = 0x0001000100010001LL;
		mb.q = 0x80007fffffff0001LL;
		movq_m2r(ma, mm0);
		pmagw_m2r(mb, mm0);
		movq_r2m(mm0, mb);
		fprintf(stdout, "pmagw: mb.q is        %08x%08x\n", mb.d[1],
			mb.d[0]);
		fprintf(stderr, "pmagw: mb.q should be 80007fff00010001\n");
		fflush(stdout); fflush(stderr);

		ma.q = 0x80007fffffff0001LL;
		mb.q = 0x0001000100010000LL;
		movq_m2r(ma, mm0);
		movq_m2r(mb, mm1);
		pmagw_r2r(mm1, mm0);
		movq_r2m(mm0, mb);
		fprintf(stdout, "pmagw: mb.q is        %08x%08x\n", mb.d[1],
			mb.d[0]);
		fprintf(stderr, "pmagw: mb.q should be 80007fff00010001\n");
		fflush(stdout); fflush(stderr);

		ma.q = 0x80007fffffff0001LL;
		mb.q = 0x0001000100010000LL;
		pmagw(ma, mb);
		fprintf(stdout, "pmagw: mb.q is        %08x%08x\n", mb.d[1],
			mb.d[0]);
		fprintf(stderr, "pmagw: mb.q should be 80007fff00010001\n");
		fflush(stdout); fflush(stderr);



		/* PADDSI ***************************************************/
		ma.q = 0x0001000100010001LL;
		mb.q = 0x80007fffffff0001LL;
		movq_m2r(ma, mm0);
		paddsiw_m2ir(mb, mm0);
		/* The saturated sum of ma and mb should now be in mm1 */
		movq_r2m(mm1, mb);
		fprintf(stdout, "paddsiw: mb.q is        %08x%08x\n", mb.d[1],
			mb.d[0]);
		fprintf(stderr, "paddsiw: mb.q should be 80017fff00000002\n");
		fflush(stdout); fflush(stderr);

		ma.q = 0x80007fffffff0001LL;
		mb.q = 0x0001000100010000LL;
		movq_m2r(ma, mm0);
		movq_m2r(mb, mm2);
		paddsiw_r2ir(mm2, mm0);
		/* The saturated sum of ma and mb should now be in mm1 */
		movq_r2m(mm1, mb);
		fprintf(stdout, "paddsw: mb.q is        %08x%08x\n", mb.d[1],
			mb.d[0]);
		fprintf(stderr, "paddsw: mb.q should be 80017fff00000001\n");
		fflush(stdout); fflush(stderr);

		ma.q = 0x80007fffffff0001LL;
		mb.q = 0x0001000100010000LL;
		paddsiw(ma, mb);
		fprintf(stdout, "paddsw: mb.q is        %08x%08x\n", mb.d[1],
			mb.d[0]);
		fprintf(stderr, "paddsw: mb.q should be 80017fff00000001\n");
		fflush(stdout); fflush(stderr);



		/* PSUBS ****************************************************/
		ma.q = 0x80007fffffff0001LL;
		mb.q = 0x0001000100010001LL;
		movq_m2r(ma, mm0);
		psubsiw_m2ir(mb, mm0);
		/* The saturated difference (ma-mb) should now be in mm1 */
		movq_r2m(mm1, mb);
		fprintf(stdout, "psubsiw: mb.q is        %08x%08x\n", mb.d[1],
			mb.d[0]);
		fprintf(stderr, "psubsiw: mb.q should be 7fff800200020000\n");
		fflush(stdout); fflush(stderr);

		ma.q = 0x0001000100010000LL;
		mb.q = 0x80007fffffff0001LL;
		movq_m2r(ma, mm0);
		movq_m2r(mb, mm2);
		psubsiw_r2ir(mm2, mm0);
		/* The saturated difference (ma-mb) should now be in mm1 */
		fprintf(stdout, "psubsiw: mb.q is        %08x%08x\n", mb.d[1],
			mb.d[0]);
		fprintf(stderr, "psubsiw: mb.q should be 80007ffefffe0001\n");
		fflush(stdout); fflush(stderr);

		ma.q = 0x807fff0101010101LL;
		mb.q = 0x01010101807fff01LL;
		psubsiw(ma, mb);
		fprintf(stdout, "psubsb: mb.q is        %08x%08x\n", mb.d[1],
			mb.d[0]);
		fprintf(stderr, "psubsb: mb.q should be 7f820200807efe00\n");
		fflush(stdout); fflush(stderr);



		/* PMULHR ***************************************************/
		ma.q = 0x8000700000ff8000LL;
		mb.q = 0x0200400000ffffffLL;
		movq_m2r(ma, mm0);
		pmulhrw_m2r(mb, mm0);
		movq_r2m(mm0, mb);
		fprintf(stdout, "pmulhrw: mb.q is        %08x%08x\n", mb.d[1],
			mb.d[0]);
		fprintf(stderr, "pmulhrw: mb.q should be fe0038000002ffff\n");
		fflush(stdout); fflush(stderr);


		ma.q = 0x8000700000ff8000LL;
		mb.q = 0x0200400000ffffffLL;
		movq_m2r(ma, mm0);
		movq_m2r(mb, mm1);
		pmulhrw_r2r(mm1, mm0);
		movq_r2m(mm0, mb);
		fprintf(stdout, "pmulhrw: mb.q is        %08x%08x\n", mb.d[1],
			mb.d[0]);
		fprintf(stderr, "pmulhrw: mb.q should be fe0038000002ffff\n");
		fflush(stdout); fflush(stderr);


		ma.q = 0x8000700000ff8000LL;
		mb.q = 0x0200400000ffffffLL;
		pmulhrw(ma, mb);
		fprintf(stdout, "pmulhrw: mb.q is        %08x%08x\n", mb.d[1],
			mb.d[0]);
		fprintf(stderr, "pmulhrw: mb.q should be fe0038000002ffff\n");
		fflush(stdout); fflush(stderr);




		/* PMULHRI **************************************************/
		ma.q = 0x8000700000ff8000LL;
		mb.q = 0x0200400000ffffffLL;
		movq_m2r(ma, mm0);
		pmulhriw_m2ir(mb, mm0);
		movq_r2m(mm1, mb);
		fprintf(stdout, "pmulhriw: mb.q is        %08x%08x\n", mb.d[1],
			mb.d[0]);
		fprintf(stderr, "pmulhriw: mb.q should be fe0038000002ffff\n");
		fflush(stdout); fflush(stderr);


		ma.q = 0x8000700000ff8000LL;
		mb.q = 0x0200400000ffffffLL;
		movq_m2r(ma, mm0);
		movq_m2r(mb, mm2);
		pmulhriw_r2ir(mm2, mm0);
		movq_r2m(mm1, mb);
		fprintf(stdout, "pmulhriw: mb.q is        %08x%08x\n", mb.d[1],
			mb.d[0]);
		fprintf(stderr, "pmulhriw: mb.q should be fe0038000002ffff\n");
		fflush(stdout); fflush(stderr);


		ma.q = 0x8000700000ff8000LL;
		mb.q = 0x0200400000ffffffLL;
		pmulhriw(ma, mb);
		fprintf(stdout, "pmulhriw: mb.q is        %08x%08x\n", mb.d[1],
			mb.d[0]);
		fprintf(stderr, "pmulhriw: mb.q should be fe0038000002ffff\n");
		fflush(stdout); fflush(stderr);



		/* PMACHRI **************************************************/
		ma.q = 0x8000700000ff8000LL;
		mb.q = 0x0200400000ffffffLL;
		mc.q = 0x0001000200030004LL;
		movq_m2r(mc, mm1);
		movq_m2r(ma, mm0);
		pmachriw_m2ir(mb, mm0);
		movq_r2m(mm1, mb);
		fprintf(stdout, "pmachriw: mb.q is        %08x%08x\n", mb.d[1],
			mb.d[0]);
		fprintf(stderr, "pmachriw: mb.q should be fe01380200050003\n");
		fflush(stdout); fflush(stderr);


		ma.q = 0x8000700000ff8000LL;
		mb.q = 0x0200400000ffffffLL;
		mc.q = 0x0001000200030004LL;
		movq_m2r(mc, mm1);
		movq_m2r(ma, mm0);
		movq_m2r(mb, mm2);
		pmachriw_r2ir(mm2, mm0);
		movq_r2m(mm1, mb);
		fprintf(stdout, "pmachriw: mb.q is        %08x%08x\n", mb.d[1],
			mb.d[0]);
		fprintf(stderr, "pmachriw: mb.q should be fe01380200050003\n");
		fflush(stdout); fflush(stderr);


		ma.q = 0x8000700000ff8000LL;
		mb.q = 0x0200400000ffffffLL;
		mc.q = 0x0001000200030004LL;
		movq_m2r(mc, mm1);
		pmachriw(ma, mb);
		fprintf(stdout, "pmachriw: mb.q is        %08x%08x\n", mb.d[1],
			mb.d[0]);
		fprintf(stderr, "pmachriw: mb.q should be fe01380200050003\n");
		fflush(stdout); fflush(stderr);



		/* PAVE *****************************************************/
		ma.q = 0x0001000100010001LL;
		mb.q = 0x80007fffffff0001LL;
		movq_m2r(ma, mm0);
		paveb_m2r(mb, mm0);
		movq_r2m(mm0, mb);
		fprintf(stdout, "paveb: mb.q is        %08x%08x\n", mb.d[1],
			mb.d[0]);
		fprintf(stderr, "paveb: mb.q should be 40003f807f800001\n");
		fflush(stdout); fflush(stderr);

		ma.q = 0x0001000100010001LL;
		mb.q = 0x80007fffffff0001LL;
		movq_m2r(ma, mm0);
		movq_m2r(mb, mm1);
		paveb_r2r(mm1, mm0);
		movq_r2m(mm0, mb);
		fprintf(stdout, "paveb: mb.q is        %08x%08x\n", mb.d[1],
			mb.d[0]);
		fprintf(stderr, "paveb: mb.q should be 40003f807f800001\n");
		fflush(stdout); fflush(stderr);

		ma.q = 0x0001000100010001LL;
		mb.q = 0x80007fffffff0001LL;
		paveb(ma, mb);
		fprintf(stdout, "paveb: mb.q is        %08x%08x\n", mb.d[1],
			mb.d[0]);
		fprintf(stderr, "paveb: mb.q should be 40003f807f800001\n");
		fflush(stdout); fflush(stderr);



		/* PDIST ****************************************************/
		ma.q = 0x808080807f7f7f7fLL;
		mb.q = 0x80007fff80007fffLL;
		mc.q = 0x0102030405060708LL;
		movq_m2r(ma, mm0);
		movq_m2r(mc, mm1);
		pdistib_m2ir(mb, mm0);
		movq_r2m(mm1, mb);
		fprintf(stdout, "pdistb: mb.q is        %08x%08x\n", mb.d[1],
			mb.d[0]);
		fprintf(stderr, "pdistb: mb.q should be 0182048306850788\n");
		fflush(stdout); fflush(stderr);

		ma.q = 0x0001000100010001LL;
		mb.q = 0x80007fffffff0001LL;
		mc.q = 0x0001000200030004LL;
		movq_m2r(mc, mm1);
		pdistib(ma, mb);
		fprintf(stdout, "pdistb: mb.q is        %08x%08x\n", mb.d[1],
			mb.d[0]);
		fprintf(stderr, "pdistb: mb.q should be 0182048306850788\n");
		fflush(stdout); fflush(stderr);



		/* PMVZ *****************************************************/
		ma.q = 0x0011A001110000A0LL;
		mb.q = 0x0102030405060708LL;
		mc.q = 0xffffffffffffffffLL;
		movq_m2r(ma, mm0);
		movq_m2r(mc, mm1);
		pmvzb_m2ir(mb, mm0);
		movq_r2m(mm1, mb);
		fprintf(stdout, "pmvzb: mb.q is        %08x%08x\n", mb.d[1],
			mb.d[0]);
		fprintf(stderr, "pmvzb: mb.q should be 01ffffffff0607ff\n");
		fflush(stdout); fflush(stderr);



		/* PMVNZ ****************************************************/
		ma.q = 0x0011A001110000A0LL;
		mb.q = 0x0102030405060708LL;
		mc.q = 0xffffffffffffffffLL;
		movq_m2r(ma, mm0);
		movq_m2r(mc, mm1);
		pmvnzb_m2ir(mb, mm0);
		movq_r2m(mm1, mb);
		fprintf(stdout, "pmvnzb: mb.q is        %08x%08x\n", mb.d[1],
			mb.d[0]);
		fprintf(stderr, "pmvnzb: mb.q should be ff02030405ffff08\n");
		fflush(stdout); fflush(stderr);



		/* PMVLZ ****************************************************/
		ma.q = 0x0011A001110000A0LL;
		mb.q = 0x0102030405060708LL;
		mc.q = 0xffffffffffffffffLL;
		movq_m2r(ma, mm0);
		movq_m2r(mc, mm1);
		pmvlzb_m2ir(mb, mm0);
		movq_r2m(mm1, mb);
		fprintf(stdout, "pmvlzb: mb.q is        %08x%08x\n", mb.d[1],
			mb.d[0]);
		fprintf(stderr, "pmvlzb: mb.q should be ffff03ffffffff08\n");
		fflush(stdout); fflush(stderr);



		/* PMVLZ ****************************************************/
		ma.q = 0x0011A001110000A0LL;
		mb.q = 0x0102030405060708LL;
		mc.q = 0xffffffffffffffffLL;
		movq_m2r(ma, mm0);
		movq_m2r(mc, mm1);
		pmvlzb_m2ir(mb, mm0);
		movq_r2m(mm1, mb);
		fprintf(stdout, "pmvlzb: mb.q is        %08x%08x\n", mb.d[1],
			mb.d[0]);
		fprintf(stderr, "pmvlzb: mb.q should be 0102ff04050607ff\n");
		fflush(stdout); fflush(stderr);



		/* Fast Exit MXX ********************************************/
		femms();
	}

	/* Clean-up and exit nicely */
	return(0);
}

/*
xmmxtest.c:87: invalid `asm': operand number missing after %-letter
xmmxtest.c:121: invalid `asm': operand number missing after %-letter
xmmxtest.c:189: invalid `asm': operand number missing after %-letter
xmmxtest.c:227: invalid `asm': operand number missing after %-letter
*/

